
#include <gdk/gdk.h>
#include "gtk/gtksignal.h"

#include "gtk/gtkadjustment.h"
#include <gtk/gtkhbox.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkentry.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtkimage.h>



#define GNOME_DB_FORM_PAGE_FLIPPER(obj)          GTK_CHECK_CAST (obj, gnome_db_form_page_flipper_get_type (), GnomeDbFormPageFlipper)
#define GNOME_DB_FORM_PAGE_FLIPPER_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gnome_db_form_page_flipper_get_type (), GnomeDbFormPageFlipperClass)
#define GNOME_DB_IS_FORM_PAGE_FLIPPER(obj)       GTK_CHECK_TYPE (obj, gnome_db_form_page_flipper_get_type ())


typedef struct _GnomeDbFormPageFlipper       GnomeDbFormPageFlipper;
typedef struct _GnomeDbFormPageFlipperClass  GnomeDbFormPageFlipperClass;

struct _GnomeDbFormPageFlipper {
	GtkHBox hbox;
  
	GtkButton *prev, *next, *new_record;
	GtkEntry  *entry;
	GtkLabel  *label1, *label2;

	GtkAdjustment *adjustment;

	gboolean (*ok_to_flip) (gpointer);
	gpointer ok_to_flip_user_data;
};

struct _GnomeDbFormPageFlipperClass {
  GtkHBoxClass parent_class;
};

/* only one public function */

GtkWidget*	gnome_db_form_page_flipper_new (GtkAdjustment *adjustment,
						gboolean (*ok_to_flip) (gpointer),
						gpointer ok_to_flip_user_data);

/* the rest are private */
GtkType        gnome_db_form_page_flipper_get_type (void);
static void    gnome_db_form_page_flipper_class_init (GnomeDbFormPageFlipperClass *klass);
static void    gnome_db_form_page_flipper_init (GnomeDbFormPageFlipper      *ttt);

void update_sensitivity (GnomeDbFormPageFlipper *flipper);
void on_adjustment_changed (GtkAdjustment *adjustment, GnomeDbFormPageFlipper *flipper);
void on_adjustment_value_changed (GtkAdjustment *adjustment, GnomeDbFormPageFlipper *flipper);
void on_entry (GtkEntry *entry, GnomeDbFormPageFlipper *flipper);
void on_prev (GtkButton *button, GnomeDbFormPageFlipper *flipper);
void on_next (GtkButton *button, GnomeDbFormPageFlipper *flipper);
void on_new_record (GtkButton *button, GnomeDbFormPageFlipper *flipper);
void try_set_value (GnomeDbFormPageFlipper *flipper, gint value);



GType
gnome_db_form_page_flipper_get_type ()
{
	static GType type = 0;

	if (!type) {
		static const GTypeInfo info = {
			sizeof (GnomeDbFormPageFlipperClass),
			NULL,
			NULL,
			(GClassInitFunc) gnome_db_form_page_flipper_class_init,
			NULL,
			NULL,
			sizeof (GnomeDbFormPageFlipper),
			0,
			(GInstanceInitFunc) gnome_db_form_page_flipper_init,
		};
		type = g_type_register_static (GTK_TYPE_HBOX, "GnomeDbFormPageFlipper", &info, 0);
	}
	return type;
}


static void
gnome_db_form_page_flipper_class_init (GnomeDbFormPageFlipperClass *class)
{
}


GtkButton* 
image_button (gchar *stock_image)
{
	GtkButton *button = (GtkButton*) gtk_button_new ();
	GtkWidget *image = gtk_image_new_from_stock (stock_image, GTK_ICON_SIZE_BUTTON);
	gtk_container_add (GTK_CONTAINER (button), image);
	return button;
}

static void
gnome_db_form_page_flipper_init (GnomeDbFormPageFlipper *flipper)
{
	flipper->ok_to_flip = NULL;
	flipper->ok_to_flip_user_data = NULL;

	flipper->prev = image_button ("gtk-go-back");
	g_signal_connect (flipper->prev, "clicked", (GCallback) on_prev, flipper);
	gtk_box_pack_start ( (GtkBox*) flipper, (GtkWidget*) flipper->prev, FALSE, FALSE, 0);

	flipper->label1 = (GtkLabel*) gtk_label_new ("Record");
	gtk_label_set_justify (GTK_LABEL (flipper->label1), GTK_JUSTIFY_LEFT);
	gtk_misc_set_padding (GTK_MISC (flipper->label1), 4, 0);
	gtk_box_pack_start ( (GtkBox*) flipper, (GtkWidget*) flipper->label1, FALSE, FALSE, 0);

	flipper->entry = (GtkEntry*) gtk_entry_new ();
	g_signal_connect (flipper->entry, "activate", (GCallback) on_entry, flipper);
	gtk_widget_set_size_request ( (GtkWidget*) flipper->entry, 36, -1);
	gtk_box_pack_start ( (GtkBox*) flipper, (GtkWidget*) flipper->entry, FALSE, TRUE, 0);


	flipper->label2 = (GtkLabel*) gtk_label_new ("of 70");
	gtk_label_set_justify (GTK_LABEL (flipper->label2), GTK_JUSTIFY_LEFT);
	gtk_misc_set_padding (GTK_MISC (flipper->label2), 4, 0);
	gtk_box_pack_start ( (GtkBox*) flipper, (GtkWidget*) flipper->label2, FALSE, FALSE, 0);

	flipper->next = image_button ("gtk-go-forward");
	g_signal_connect (flipper->next, "clicked", (GCallback) on_next, flipper);
	gtk_box_pack_start ( (GtkBox*) flipper, (GtkWidget*) flipper->next, FALSE, FALSE, 0);

	flipper->new_record = (GtkButton*) gtk_button_new_from_stock ("gtk-new");
	g_signal_connect (flipper->new_record, "clicked", (GCallback) on_new_record, flipper);
	gtk_box_pack_start ( (GtkBox*) flipper, (GtkWidget*) flipper->new_record, FALSE, FALSE, 0);

	gtk_box_pack_start ( (GtkBox*) flipper, (GtkWidget*) gtk_hseparator_new (), TRUE, TRUE, 0);

	gtk_widget_show_all ( (GtkWidget*) flipper);

	flipper->adjustment = NULL;
}


/**
 * default_range_new
 *
 * Create a nice range widget for use in as a record fliper in non-continuous mode
 */
GtkWidget*
gnome_db_form_page_flipper_new (GtkAdjustment *adjustment,
				gboolean (*ok_to_flip) (gpointer),
				gpointer ok_to_flip_user_data)
{
	g_return_val_if_fail (GTK_IS_ADJUSTMENT (adjustment), NULL);

	GnomeDbFormPageFlipper *flipper = (GnomeDbFormPageFlipper*) 
		  g_object_new (gnome_db_form_page_flipper_get_type (), NULL);

	flipper->ok_to_flip = ok_to_flip;
	flipper->ok_to_flip_user_data = ok_to_flip_user_data;

	g_signal_connect (adjustment,
			  "changed",
			  (GCallback) on_adjustment_changed,
			  flipper);
	g_signal_connect (adjustment,
			  "value-changed",
			  (GCallback) on_adjustment_value_changed,
			  flipper);

	flipper->adjustment = adjustment;

	on_adjustment_changed (adjustment, flipper);
	on_adjustment_value_changed (adjustment, flipper);
	update_sensitivity (flipper);

	return (GtkWidget*) flipper;
}
/*
GtkWidget*
gnome_db_form_page_flipper_new___TEST (GtkAdjustment *adjustment)
{
	GtkWidget *scale = gtk_hscale_new (adjustment);
	gtk_scale_set_digits ( (GtkScale*) scale, 0);
	return scale;
}
*/



/* =========================================== */



void
update_sensitivity (GnomeDbFormPageFlipper *flipper)
{
	g_assert (GNOME_DB_IS_FORM_PAGE_FLIPPER (flipper));

	GtkAdjustment *a = flipper->adjustment;

	gtk_widget_set_sensitive ( (GtkWidget*) flipper->prev, 
				   a->lower < a->value);
	gtk_widget_set_sensitive ( (GtkWidget*) flipper->next, 
				   a->value < a->upper);
	gtk_widget_set_sensitive ( (GtkWidget*) flipper->new_record, 
				   a->lower == -1);
}


void
on_adjustment_changed (GtkAdjustment *adjustment, GnomeDbFormPageFlipper *flipper)
{
	g_printerr ("\nadjustment changed");
	g_assert (GTK_IS_ADJUSTMENT (adjustment));
	g_assert (GNOME_DB_IS_FORM_PAGE_FLIPPER (flipper));

	gchar *s = g_strdup_printf ("of %0.0f", adjustment->upper+1);
	gtk_label_set_text (flipper->label2, s);
	g_free (s);

	update_sensitivity (flipper);
}


void
on_adjustment_value_changed (GtkAdjustment *adjustment, GnomeDbFormPageFlipper *flipper)
{
	g_printerr ("\nadjustment val changed");
	g_assert (GTK_IS_ADJUSTMENT (adjustment));
	g_assert (GNOME_DB_IS_FORM_PAGE_FLIPPER (flipper));

	if (adjustment->value == -1) 
		gtk_entry_set_text (flipper->entry, "new");
	else {
		gchar *s = g_strdup_printf ("%0.0f", adjustment->value+1);
		gtk_entry_set_text (flipper->entry, s);
		g_free (s);
	}
			    
	update_sensitivity (flipper);
}


void
on_entry (GtkEntry *entry, GnomeDbFormPageFlipper *flipper)
{
	g_assert (GNOME_DB_IS_FORM_PAGE_FLIPPER (flipper));

	const gchar *s = gtk_entry_get_text (flipper->entry);
	gint n = (gint) (0.5 + g_strtod (s, NULL));
	try_set_value (flipper, n-1);
}


void
on_prev (GtkButton *button, GnomeDbFormPageFlipper *flipper)
{
	g_assert (GNOME_DB_IS_FORM_PAGE_FLIPPER (flipper));
	try_set_value (flipper, flipper->adjustment->value - 1);
}


void
on_next (GtkButton *button, GnomeDbFormPageFlipper *flipper)
{
	g_assert (GNOME_DB_IS_FORM_PAGE_FLIPPER (flipper));
	try_set_value (flipper, flipper->adjustment->value + 1);
}


void
on_new_record (GtkButton *button, GnomeDbFormPageFlipper *flipper)
{
	g_assert (GNOME_DB_IS_FORM_PAGE_FLIPPER (flipper));
	try_set_value (flipper, -1);
}

void
try_set_value (GnomeDbFormPageFlipper *flipper, gint value)
{
	if (! flipper->ok_to_flip ||
	    flipper->ok_to_flip (flipper->ok_to_flip_user_data))
		gtk_adjustment_set_value (flipper->adjustment, value);
}
