
/* 
#include <libgnomedb/gnome-db-list.h>
#include <libgnomedb/gnome-db-util.h>
*/
/* #include "gnome-db-i18n.h"  */

#include "gnome-db-form.h"

/*
 * convinient functions for binding various widgets 
 */



gboolean
toggle_button_get (const GtkWidget* widget, GdaValue* value)
{
	g_return_if_fail (value);
	g_assert (GTK_IS_TOGGLE_BUTTON (widget));

	if (value->type != GDA_VALUE_TYPE_BOOLEAN)  return FALSE;
	gda_value_set_boolean (value, gtk_toggle_button_get_active ( (GtkToggleButton*) widget));
	return TRUE;
}

void
toggle_button_set (GtkWidget* widget, const GdaValue* value)
{
	g_return_if_fail (value);
	g_assert (GTK_IS_TOGGLE_BUTTON (widget));

	gtk_toggle_button_set_active ( (GtkToggleButton*) widget,
				       gda_value_get_boolean(value));
}

void
gnome_db_form_slot_bind_toggle_button (GnomeDbFormSlot *form_slot, const gchar* column, 
				       GtkToggleButton *toggle_button)
{
	g_return_if_fail (GTK_IS_TOGGLE_BUTTON (toggle_button));
	gnome_db_form_slot_bind (form_slot, column, toggle_button,
				 toggle_button_get,
				 toggle_button_set,
				 gtk_widget_set_sensitive);

	g_signal_connect_swapped ( toggle_button, 
				   "toggled", 
				   (GCallback) gnome_db_form_slot_set_edited, 
				   form_slot);
}


/* ================ */

gboolean
entry_get (const GtkWidget* widget, GdaValue* value) {
	g_return_if_fail (value);
	g_assert (GTK_IS_ENTRY (widget));

	const gchar *str = gtk_entry_get_text ( (GtkEntry*) widget);

	if (gda_value_set_from_string (value, str, value->type))
		return TRUE;
	if (strlen(str)==0) {
		gda_value_set_null (value);
		return TRUE;
	}
	g_printerr ("\ncan't parse %s into %d", str, value->type);
	return FALSE;
}

void
entry_set (GtkWidget* widget, const GdaValue* value) {
	g_return_if_fail (value);
	g_assert (GTK_IS_ENTRY (widget));


	if (gda_value_get_type (value) == GDA_VALUE_TYPE_NULL) 
		gtk_entry_set_text ( (GtkEntry*) widget, "");
	else {
		gchar *str = gda_value_stringify (value);
		gtk_entry_set_text ( (GtkEntry*) widget, str);
		g_free (str);
	}

	/* just for debugging blob
	int i;
	for (i=0; 1; i++)  if (str[i]=='\0') {
		if (str[i-1]=='o' && str[i-2]=='l')  str[i]='#';
		else break;
	}
	*/
}

void
gnome_db_form_slot_bind_entry (GnomeDbFormSlot *form_slot, const gchar* column, 
			       GtkEntry *entry)
{
	g_return_if_fail (GTK_IS_ENTRY (entry));

	gnome_db_form_slot_bind (form_slot, column, entry,
				 entry_get,
				 entry_set,
				 (SET_EDITABLE) gtk_entry_set_editable);

	g_signal_connect_swapped ( entry, 
				   "changed",
				   (GCallback) gnome_db_form_slot_set_edited, 
				   form_slot);

	g_signal_connect_swapped ( entry, 
				   "activate", 
				   (GCallback) gnome_db_form_slot_save, 
				   form_slot);
}


/* ================ */


gboolean
text_view_get (const GtkWidget* widget, GdaValue* value)
{
	g_return_if_fail (value);
	g_assert (GTK_IS_TEXT_VIEW (widget));

	GtkTextIter begin, end;
	GtkTextBuffer *buffer = gtk_text_view_get_buffer ( (GtkTextView*) widget);
	gtk_text_buffer_get_bounds (buffer, &begin, &end);
	gchar *str = gtk_text_buffer_get_text (buffer, &begin, &end, TRUE);
	gboolean success = gda_value_set_from_string (value, str, value->type);
	g_free (str);
	return success;
}

void
text_view_set (GtkWidget* widget, const GdaValue* value)
{
	g_return_if_fail (value);
	g_assert (GTK_IS_TEXT_VIEW (widget));

	gchar* str = gda_value_stringify (value);
	GtkTextBuffer *buffer = gtk_text_view_get_buffer ( (GtkTextView*) widget);
	gtk_text_buffer_set_text (buffer, str, -1);
	g_free (str);
}
void
gnome_db_form_slot_bind_text_view (GnomeDbFormSlot *form_slot, const gchar* column, 
				   GtkTextView *entry)
{
	g_return_if_fail (GTK_IS_TEXT_VIEW (entry));
	gnome_db_form_slot_bind (form_slot, column, entry,
				 text_view_get,
				 text_view_set,
				 (SET_EDITABLE) gtk_text_view_set_editable);
	// FIXME, connect slot_[set_edited | save] signals
}

