/* GNOME DB library
 * Copyright (C) 1999-2002 The GNOME Foundation.
 *
 * AUTHORS:
 *      Rodrigo Moya <rodrigo@gnome-db.org>
 *
 * This Library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this Library; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <gtk/gtkbbox.h>
#include <gtk/gtkenums.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkstock.h>
#include <gtk/gtktable.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtkentry.h>
#include <gtk/gtktextview.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtkmessagedialog.h>

#include <gtk/gtkstyle.h>

/* 
#include <libgnomedb/gnome-db-list.h>
#include <libgnomedb/gnome-db-util.h>
*/
/* #include "gnome-db-i18n.h"  */

#include "gnome-db-form.h"
#include "gnome-db-form-private.h"


static const int BORDER_WIDTH = 4;
static GObjectClass *parent_class = NULL;

struct _GnomeDbFormSlotPrivate {
	GnomeDbForm    *form;
        gint		row;
        gboolean	edited;
	GHashTable     *object_hash;  // data bound object
	GList	       *object_list;  // not bound
};

enum {
	SYNC_WIDGETS,  // FIXME rename to SYNC
	SAVE,  //	rename to SAVE
	EDIT,
	LAST_SIGNAL
};
static gint form_slot_signals[LAST_SIGNAL];

typedef struct {
	gpointer	       instance;
	gchar		      *column_name;
	/* gboolean	         updated; */
	GET_WIDGET_GDA_VALUE   get_value;
	SET_WIDGET_GDA_VALUE   set_value;
	SET_EDITABLE	       set_editable;
} BoundObjectInfo;


/* privates fucntions */
static void	gnome_db_form_slot_class_init (GnomeDbFormSlotClass *klass);
static void	gnome_db_form_slot_init (GnomeDbFormSlot *form_slot);
static void	gnome_db_form_slot_finalize (GObject *object);
GType		gnome_db_form_slot_get_type (void);

GtkWidget*	gnome_db_form_slot_new (GnomeDbForm* form, gint row);
gint		gnome_db_form_slot_get_row_number (GnomeDbFormSlot *form_slot);

void		gnome_db_form_slot_sync_widgets (GnomeDbFormSlot *form_slot);
void		gnome_db_form_slot_set_row (GnomeDbFormSlot *form_slot, gint row);

void		sync_widgets_callback (GnomeDbFormSlot *form_slot);
void		save_callback (GnomeDbFormSlot *form_slot);

void		edit_callback (GnomeDbFormSlot *form_slot);

void		draw_border (GnomeDbFormSlot *form_slot, GdkRectangle *area);

static gboolean form_slot_expose (GtkWidget *widget, GdkEventExpose *event);

/* ================ */

static void
gnome_db_form_slot_class_init (GnomeDbFormSlotClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);

	klass->default_sync_widgets = sync_widgets_callback;
        klass->save = save_callback;
	klass->edit = edit_callback;
	klass->null = NULL;

	((GtkWidgetClass *) klass)->expose_event = form_slot_expose;

	form_slot_signals[SYNC_WIDGETS] =
		g_signal_new ("sync_widgets",
			      G_TYPE_FROM_CLASS (object_class),
			      G_SIGNAL_RUN_LAST,
			      G_STRUCT_OFFSET (GnomeDbFormSlotClass, default_sync_widgets),
			      NULL, NULL,
			      g_cclosure_marshal_VOID__VOID,
			      G_TYPE_NONE, 0);
	form_slot_signals[SAVE] =
		g_signal_new ("save",
			      G_TYPE_FROM_CLASS (object_class),
			      G_SIGNAL_RUN_LAST,
			      G_STRUCT_OFFSET (GnomeDbFormSlotClass, save),
			      NULL, NULL,
			      g_cclosure_marshal_VOID__VOID,
			      G_TYPE_NONE, 0);
	form_slot_signals[EDIT] =
		g_signal_new ("edit",
			      G_TYPE_FROM_CLASS (object_class),
			      G_SIGNAL_RUN_LAST,
			      G_STRUCT_OFFSET (GnomeDbFormSlotClass, edit),
			      NULL, NULL,
			      g_cclosure_marshal_VOID__VOID,
			      G_TYPE_NONE, 0);

}


static void
gnome_db_form_slot_init (GnomeDbFormSlot *form_slot)
{
	g_return_if_fail (GNOME_DB_IS_FORM_SLOT (form_slot));

	form_slot->priv			= g_new (GnomeDbFormSlotPrivate, 1);
	form_slot->priv->form		= NULL;
	form_slot->priv->row		= 0;
	form_slot->priv->edited		= FALSE;
	form_slot->priv->object_hash    = NULL;
	form_slot->priv->object_list	= NULL;
}


void
helper1 (gpointer key, BoundObjectInfo *value, gpointer *user_data)
{
	// no g_delete? 
	// g_renew (void, value, 0);
	g_free (value->column_name);
	g_free (value);
}
void
helper2 (gpointer value, gpointer user_data) 
{
	// no g_delete? 
	// g_renew (void, value, 0);
	g_free (value);
}
static void
gnome_db_form_slot_finalize (GObject *object)
{
	GnomeDbFormSlot *form = (GnomeDbFormSlot *) object;
	g_return_if_fail (GNOME_DB_IS_FORM_SLOT (form));

	if (form->priv->edited) {
		// FIXME tell data_model that we are no longer editing that GdaRow 
	}
	g_hash_table_foreach (form->priv->object_hash, (GHFunc) helper1, NULL);
	g_hash_table_destroy (form->priv->object_hash);
	g_list_foreach (form->priv->object_list, helper2, NULL);
	g_list_free (form->priv->object_list);
	g_free (form->priv);

	/* chain to parent class */
	parent_class->finalize (object);
}


GType
gnome_db_form_slot_get_type (void)
{
        static GType type = 0;
        if (!type) {
                static const GTypeInfo info = {
                	sizeof (GnomeDbFormSlotClass),
                	NULL, /* base_init */
                        NULL, /* base_finalize */
                	(GClassInitFunc) gnome_db_form_slot_class_init,
                        NULL, /* class_finalize */
                	NULL, /* class_data */
                        sizeof (GnomeDbFormSlot),
                	0,
                	(GInstanceInitFunc) gnome_db_form_slot_init,
                };
                type = g_type_register_static (GTK_TYPE_VBOX, "GnomeDbFormSlot", &info, 0);
        }
        return type;
}



void
set_focus_child_callback (GnomeDbFormSlot *form_slot, GtkWidget *widget, gpointer user_data)
{
	g_assert (GNOME_DB_IS_FORM_SLOT (form_slot));
	if (! form_slot->priv->edited)  return;
	
	if (widget==NULL && ! gtk_widget_is_focus ((GtkWidget*) form_slot) ) {
		g_printerr ("\nFIXME: show warning balloon::unsaved data.");
	}
}
GtkWidget*
gnome_db_form_slot_new (GnomeDbForm* form, gint row)
{
        GnomeDbFormSlot* slot = g_object_new (gnome_db_form_slot_get_type (), NULL);
	slot->priv->form = form;
	slot->priv->row  = row;
	slot->priv->object_hash = g_hash_table_new (g_str_hash, g_str_equal);

	((GtkContainer*) slot)->border_width = BORDER_WIDTH;
	// ((GtkBox*) slot)->spacing = 0;
	((GtkBox*) slot)->homogeneous = 0;

	g_signal_connect ( (GtkVBox*) slot, "set-focus-child", (GCallback) set_focus_child_callback, NULL);
	return GTK_WIDGET (slot);
}


/* == member functions ============== */


GnomeDbForm*
gnome_db_form_slot_get_form (GnomeDbFormSlot *form_slot)
{
	return form_slot->priv->form;
}

extern GdaDataModel* gnome_db_form_get_model (GnomeDbForm *form);
GdaDataModel*
gnome_db_form_slot_get_data_model (GnomeDbFormSlot *form_slot)
{
	return gnome_db_form_get_model (form_slot->priv->form);
}

gint
gnome_db_form_slot_get_row_number (GnomeDbFormSlot *form_slot)
{
	return form_slot->priv->row;
}

const GdaRow*
gnome_db_form_slot_get_gda_row (GnomeDbFormSlot *form_slot)
{
	return gda_data_model_get_row (gnome_db_form_slot_get_data_model (form_slot),
				       form_slot->priv->row);
}


void
gnome_db_form_slot_bind (GnomeDbFormSlot *form_slot, 
			 const gchar* column, 
			 void *instance,
			 GET_WIDGET_GDA_VALUE get_value, 
			 SET_WIDGET_GDA_VALUE set_value,
			 SET_EDITABLE set_editable)
{
	g_return_if_fail (GNOME_DB_IS_FORM_SLOT (form_slot));

	BoundObjectInfo *t = g_new (BoundObjectInfo, 1);
	t->column_name = g_strdup (column);

	if (column==NULL || g_strcasecmp(column, "")==0) {
		form_slot->priv->object_list = g_list_append (form_slot->priv->object_list, t);
	} else {
		gpointer val = g_hash_table_lookup (form_slot->priv->object_hash, column);
		g_printerr ("\nlooking up %s", column);
		if (val) {
			g_printerr ("gnome_db_form_slot_bind(): Column \"%s\" has already been binded to another widget\n", column);
			form_slot->priv->object_list = g_list_append (form_slot->priv->object_list, t);
		} else {
			g_printerr ("--binding");
			/* Tough hash is compared by content, but the key pointer isn't copied.
			   So we have to g_strdup() above and use t->column_name here.  Since
			   "column" can be destroyed by users anytime.
			*/
			g_hash_table_insert( form_slot->priv->object_hash, t->column_name, t);
		}
	}

	t->instance = instance;
	/* t->updated = FALSE; */
	t->get_value = get_value;
	t->set_value = set_value;
	t->set_editable = set_editable;
}


/* == data syncing code ================ */
        
void
gnome_db_form_slot_sync_widgets (GnomeDbFormSlot *form_slot)
{
	g_return_if_fail (GNOME_DB_IS_FORM_SLOT (form_slot));

	gtk_signal_emit ( (GtkObject*) form_slot, form_slot_signals[SYNC_WIDGETS]);
}

void
gnome_db_form_slot_set_row (GnomeDbFormSlot *form_slot, gint row)
{
	g_assert (GNOME_DB_IS_FORM_SLOT (form_slot));

	form_slot->priv->row = row;
	gnome_db_form_slot_sync_widgets (form_slot);
}


typedef void (*SET_BOOL) (gpointer instance, gboolean value);
void
sync_widgets_callback (GnomeDbFormSlot *form_slot)
{
	g_assert (GNOME_DB_IS_FORM_SLOT (form_slot));

	GdaDataModel *model = gnome_db_form_slot_get_data_model(form_slot);
	if (model==NULL)  return;
	g_assert (GDA_IS_DATA_MODEL (model));

	const GdaRow *row = gnome_db_form_slot_get_gda_row(form_slot);

	gint cols = gda_data_model_get_n_columns (model);
	gint i;
	for (i=0; i<cols; i++) {
		GdaFieldAttributes *fa = gda_data_model_describe_column (model, i);
		const gchar *name = gda_field_attributes_get_name (fa);
		BoundObjectInfo *info = g_hash_table_lookup (form_slot->priv->object_hash, name);
		g_printerr("\nentry %s", name);
		if (info) {
			g_printerr("--syncing");
			g_signal_handlers_block_by_func (info->instance, 
							 gnome_db_form_slot_set_edited,
							 form_slot);
			info->set_value (info->instance, gda_row_get_value (row, i));
			g_signal_handlers_unblock_by_func (info->instance, 
							   gnome_db_form_slot_set_edited,
							   form_slot);
		}
	}
	/*
	GList *list = form_slot->priv->object_list;
	GdaValue *blank = gda_value_new_null();
	while (list) {
		BoundObjectInfo *info = list->data;
		info->set_value (info->instance, blank);
		list = list->next;
	}
	g_free (blank);
	*/
}

/* == updating data ============== */


/* save the data
 * retern whether the data was saved successfully
 */
gboolean gnome_db_form_slot_save (GnomeDbFormSlot *form_slot)
{
	g_printerr ("\nform_slot_save");
	g_return_if_fail (GNOME_DB_IS_FORM_SLOT (form_slot));

	if (! form_slot->priv->edited)  return TRUE;
	gtk_signal_emit ( (GtkObject*) form_slot, form_slot_signals[SAVE]);
	return ! form_slot->priv->edited;
}

void
save_callback (GnomeDbFormSlot *form_slot)
{
	g_assert (GNOME_DB_IS_FORM_SLOT (form_slot));
	if (! form_slot->priv->edited)  return;

	g_printerr ("\nSave time!");

	GdaDataModel *model = gnome_db_form_get_model (form_slot->priv->form);
	GdaRow *row = gda_data_model_get_updatable_row (model, form_slot->priv->row);

	g_printerr ("\nlet's see");
	
	if (row) {
		g_printerr (" -- in");
		gint cols = gda_data_model_get_n_columns (model);
		// FIXME  backup each GdaValue
		gint i;
		for (i=0; i<cols; i++) {
			GdaFieldAttributes *fa = gda_data_model_describe_column (model, i);
			const gchar *name = gda_field_attributes_get_name (fa);
			BoundObjectInfo *info = g_hash_table_lookup (form_slot->priv->object_hash, name);
			g_printerr("\nentry %s", name);
			if (info) {
				GdaValue *value = gda_row_get_updatable_value (row, i);
				// value->type could be TYPE_NULL, but get_value() relies
				// on this to tell the type.  hence this ugly hack.
				// 
				// NULL in libgda should be value, not type!
				value->type = get_type (form_slot->priv->form, i);
				// break if GdaValue can't be set
				if (! (info->get_value (info->instance, value)))
				       break;
				g_printerr(" -- saved");
			}
		}
		
		if (i==cols && gda_data_model_update_row (model, row)) {
			form_slot->priv->edited = FALSE;
			draw_border (form_slot, NULL);
			return;
		}
		// FIXME  roll back to old GdaValue
	}

	GtkDialog *dialog;
	dialog = (GtkDialog*) gtk_dialog_new_with_buttons (
		"Record couldn't be saved.",
		//(GtkWindow*) gtk_widget_get_parent_window ( (GtkWidget*) form_slot->priv->form),
		NULL,
		// GTK_DIALOG_MODAL,
		0,
		"Continue editing",
		1,
		"Discard changes",
		2,
		NULL);
	
	GtkWidget *label = gtk_label_new ("The record you edited couldn't be saved.");
	gtk_container_add (GTK_CONTAINER (dialog->vbox), 
			   label);
	gtk_widget_show (label);
	gint response = gtk_dialog_run (dialog);
	gtk_widget_destroy ( (GtkWidget*) dialog);
	switch (response) {
	case 1: // "Continue editing"
		// don't let user's clean up callback run
		g_signal_stop_emission_by_name (form_slot, "save");
		break;
	case 2: // "Discard changes"
		form_slot->priv->edited = FALSE;
		draw_border (form_slot, NULL);
		gnome_db_form_slot_sync_widgets (form_slot);
		break;
	}
	// g_printerr (" and the answer is ... %d", response);
}


void
edit_callback (GnomeDbFormSlot *form_slot)
{
	g_return_if_fail (GNOME_DB_IS_FORM_SLOT (form_slot));
	g_printerr ("\nedit_callback");

	form_slot->priv->edited = TRUE;
	draw_border (form_slot, NULL);
}

void
gnome_db_form_slot_set_edited (GnomeDbFormSlot *form_slot)
{
	g_return_if_fail (GNOME_DB_IS_FORM_SLOT (form_slot));
//	g_printerr ("\nduh");
	if (form_slot->priv->edited) return;
	gtk_signal_emit ( (GtkObject*) form_slot, form_slot_signals[EDIT]);
}

gboolean
gnome_db_form_slot_get_edited (GnomeDbFormSlot *form_slot)
{
	g_return_if_fail (GNOME_DB_IS_FORM_SLOT (form_slot));
	return form_slot->priv->edited;
}





void helper_hash (gpointer key, gpointer value, gpointer user_data) {
	BoundObjectInfo *info = (BoundObjectInfo*) value;
	info->set_editable (info->instance, (gboolean) user_data);
}
void helper_list (gpointer data, gpointer user_data) {
	BoundObjectInfo *info = (BoundObjectInfo*) data;
	info->set_editable (info->instance, (gboolean) user_data);
}
void
gnome_db_form_slot_set_editable (GnomeDbFormSlot *form_slot, gboolean editable)
{
	g_return_if_fail (GNOME_DB_IS_FORM_SLOT (form_slot));
	g_hash_table_foreach (form_slot->priv->object_hash, helper_hash, (gpointer) editable);
	g_list_foreach (form_slot->priv->object_list, helper_list, (gpointer) editable);
}



// == Expose ==============================================================

/* r     : GdkRectangle r[4];
 *         Fill r with rectangle for drawing rectangular border
 * o     : Original rectangle
 * thick : Line thickness -- growing in from 'orig'.
 */
void
expose_helper (GdkRectangle *r, GdkRectangle *o, gint thick)
{
	r[0].x = o->x;
	r[0].y = o->y;
	r[0].width = o->width;
	r[0].height = thick;

	r[1].x = o->x;
	r[1].y = o->y + o->height - thick;
	r[1].width = o->width;
	r[1].height = thick;

	r[2].x = o->x;
	r[2].y = o->y;
	r[2].width = thick;
	r[2].height = o->height;

	r[3].x = o->x + o->width - thick;
	r[3].y = o->y;
	r[3].width = thick;
	r[3].height = o->height;

}

void
draw_border (GnomeDbFormSlot *form_slot, GdkRectangle *area)
{
	g_assert (GNOME_DB_IS_FORM_SLOT (form_slot));
	if (!GTK_WIDGET_DRAWABLE (form_slot))  return;

	GtkWidget *widget = (GtkWidget*) form_slot;
	GdkRectangle r[4];
	expose_helper (r, & widget->allocation, BORDER_WIDTH);
	gint i;
	for (i=0; i<4; i++)
		gtk_paint_box (widget->style, widget->window,
			       form_slot->priv->edited ? GTK_STATE_SELECTED 
						       : GTK_STATE_NORMAL,
			       GTK_SHADOW_NONE,
			       area ? area : &widget->allocation, 
			       widget, "what for?",
			       // what is that string for?
			       r[i].x, r[i].y, r[i].width, r[i].height);
}

static gboolean
form_slot_expose (GtkWidget *widget, GdkEventExpose *event)
{
	g_return_if_fail (GNOME_DB_IS_FORM_SLOT (widget));

	if (GTK_WIDGET_DRAWABLE (widget)) {
		draw_border ((GnomeDbFormSlot*) widget, &event->area);
		(* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
	}

	return FALSE;
}
