/* GNOME DB library
 * Copyright (C) 1999-2002 The GNOME Foundation.
 *
 * AUTHORS:
 *      Rodrigo Moya <rodrigo@gnome-db.org>
 *
 * This Library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this Library; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtkvbox.h>
#include <gtk/gtkscrolledwindow.h>
#include <gtk/gtkhscale.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtklabel.h>
#include <gtk/gtktable.h>
#include <gtk/gtkhseparator.h>
#include <gtk/gtkcheckbutton.h>


/* 
#include <libgnomedb/gnome-db-list.h>
#include <libgnomedb/gnome-db-util.h>
*/
/* #include "gnome-db-i18n.h"  */
#include "gnome-db-form.h"
#include "gnome-db-form-private.h"


#define PARENT_TYPE GTK_TYPE_VBOX
static GObjectClass *parent_class = NULL;

enum {
	MODEL_CHANGED,
	EDITED,
	LAST_SIGNAL
};
static gint form_signals[LAST_SIGNAL];

struct _GnomeDbFormPrivate {
	GdaDataModel   *data_model;

	GdaFieldAttributes **field_attributes;	// ugly hack to work around NULL being
						// type, not value
	FORM_TYPE       form_type;
	gboolean        continuous;
	gboolean	editable;
	gboolean	insertable;

	GtkContainer   *display_widget;  /* for later when VBox is demoted
					    from parent to just member */
	GtkContainer   *slot_container;  /* continuous mode: VBox holding the slots
					    non-continuous : the slot */
	GtkWidget      *page_flipper;	 /* keep a ref on this */
	GtkAdjustment  *adjustment;	 /* adjustment associated with page_flipper */

	gulong		gdm_changed_handler_id;
	gulong		adjustment_value_changed_handler_id;
};


/* private functions */
static void	gnome_db_form_finalize (GObject *object);
static void	gnome_db_form_class_init (GnomeDbFormClass *klass);
static void	gnome_db_form_init (GnomeDbForm *form);
GType		gnome_db_form_get_type (void);
GtkWidget      *gnome_db_form_new (void);
GtkWidget      *gnome_db_form_new_with_args (GdaDataModel *model,
					     FORM_TYPE form_type,
					     gboolean continuous);

void		gnome_db_form_sync_widgets (GnomeDbForm *form);
void		destroy_displaying_widgets (GnomeDbForm *form);
void		create_displaying_widgets (GnomeDbForm *form);

void		link_model (GnomeDbForm *form, GdaDataModel *model);
void		unlink_model (GnomeDbForm *form);

void		_GNOME_DB_FORM_TYPE_NAVIGATOR (GnomeDbFormSlot *form_slot);
void		_GNOME_DB_FORM_TYPE_LIST (GnomeDbFormSlot *form_slot);
const FORM_TYPE GNOME_DB_FORM_TYPE_NAVIGATOR =  & _GNOME_DB_FORM_TYPE_NAVIGATOR;
const FORM_TYPE GNOME_DB_FORM_TYPE_LIST =  & _GNOME_DB_FORM_TYPE_LIST;

gint		rint (gdouble d) {return (gint) 0.5+d;}



/* ================================================================
 * GnomeDbForm class implementation
 */

static void
gnome_db_form_finalize (GObject *object)
{
	g_return_if_fail (GNOME_DB_IS_FORM (object));
	GnomeDbForm *form = (GnomeDbForm *) object;

	// This saves page_flipper and adjustment from being destroyed
	// in case the user still have reference on it
	destroy_displaying_widgets (form);
	g_object_unref (form->priv->page_flipper);
	if (GTK_IS_WIDGET (form->priv->display_widget))
		gtk_widget_destroy ( (GtkWidget*) form->priv->display_widget);
	/* after VBox got demoted from parent to member
	g_object_unref(...)
	*/

	unlink_model (form);

	g_free (form->priv);
	form->priv = NULL;

	// chain to parent class
	parent_class->finalize (object);
}


static void
gnome_db_form_class_init (GnomeDbFormClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);

        /* what for? 
	object_class->set_property = gnome_db_form_set_property;
	object_class->get_property = gnome_db_form_get_property;
        */
	object_class->finalize = gnome_db_form_finalize;

	klass->model_changed = NULL;
	klass->edited = NULL;

	/* add class signals */
	form_signals[MODEL_CHANGED] =
		g_signal_new ("model_changed",
			      G_TYPE_FROM_CLASS (object_class),
			      G_SIGNAL_RUN_LAST,
			      G_STRUCT_OFFSET (GnomeDbFormClass, model_changed),
			      NULL, NULL,
			      g_cclosure_marshal_VOID__VOID,
			      G_TYPE_NONE, 0);
	form_signals[EDITED] =
		g_signal_new ("edited",
			      G_TYPE_FROM_CLASS (object_class),
			      G_SIGNAL_RUN_LAST,
			      G_STRUCT_OFFSET (GnomeDbFormClass, edited),
			      NULL, NULL,
			      g_cclosure_marshal_VOID__VOID,
			      G_TYPE_NONE, 0);
}


static void
gnome_db_form_init (GnomeDbForm *form)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));

	/* allocate private structure */
	form->priv		   = g_new0 (GnomeDbFormPrivate, 1);
	form->priv->data_model	   = NULL;
	form->priv->field_attributes = NULL;

	form->priv->form_type	   = NULL;
	form->priv->continuous     = FALSE;
	form->priv->editable	   = FALSE;
	form->priv->insertable	   = FALSE;

	form->priv->display_widget = NULL;
	form->priv->slot_container = NULL;
	form->priv->page_flipper   = NULL;
	form->priv->adjustment	   = NULL;

	form->priv->gdm_changed_handler_id = 0;
	form->priv->adjustment_value_changed_handler_id = 0;
}


GType
gnome_db_form_get_type (void)
{
	static GType type = 0;

	if (!type) {
		static const GTypeInfo info = {
			sizeof (GnomeDbFormClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) gnome_db_form_class_init,
			NULL,
			NULL,
			sizeof (GnomeDbForm),
			0,
			(GInstanceInitFunc) gnome_db_form_init
		};
		type = g_type_register_static (PARENT_TYPE, "GnomeDbForm", &info, 0);
	}
	return type;
}


typedef gboolean (*HELPER) (gpointer);
/**
 * gnome_db_form_new
 *
 * Create a new #GnomeDbForm widget, which is a high-level widget which
 * lets you display recordsets in many different ways, such as a list,
 * record-by-record, etc.
 *
 * Returns: the newly created widget.
 */
GtkWidget *
gnome_db_form_new (void)
{
	GnomeDbForm *form;

	form = g_object_new (GNOME_DB_TYPE_FORM, NULL);
	/* page_flipper (and the associated adjustment) have to exist before data_model
	   or display_widget are added in order to have signal registered */
	GtkAdjustment *adjustment = (GtkAdjustment*) gtk_adjustment_new (0, 0, 0, 1.0, 1.0, 0);
	gnome_db_form_add_page_flipper (
		form, 
		gnome_db_form_page_flipper_new(adjustment, 
					       (HELPER) gnome_db_form_save_all, 
					       form),
		adjustment);
	gnome_db_form_set_form_type (form, GNOME_DB_FORM_TYPE_NAVIGATOR);

	gtk_widget_show_all ( (GtkWidget*) form);	 

	return GTK_WIDGET (form);
}


GtkWidget
*gnome_db_form_new_with_args (GdaDataModel *model,
			      FORM_TYPE form_type,
			      gboolean continuous)
{
	/* FIXME */
}



/* == private functions ============================================== */


/* private function from gnome-db-form-slot.c */
extern void
gnome_db_form_slot_set_row (GnomeDbFormSlot *form_slot, gint row);


void
row_change_callback (GtkAdjustment *adjustment, GnomeDbFormSlot *form_slot)
{
	g_assert (GNOME_DB_IS_FORM_SLOT (form_slot));

	gdouble value = gtk_adjustment_get_value (adjustment);
	gnome_db_form_slot_set_row (form_slot, rint(value));
}


GnomeDbFormSlot *form_slot_new (GnomeDbForm *form, gint row) {
	GnomeDbFormSlot *slot = (GnomeDbFormSlot*) gnome_db_form_slot_new (form, row);
	form->priv->form_type (slot);
	gnome_db_form_slot_set_editable (slot, form->priv->editable);
	return slot;
}
/* create the new form according to form_type and continuous values */
void
create_displaying_widgets (GnomeDbForm *form)
{
	g_printerr ("\ncreate_displaying_widgets() ");

	g_assert (GNOME_DB_IS_FORM (form));
	g_assert (form->priv->display_widget == NULL);
	g_assert (form->priv->slot_container == NULL);

	if (form->priv->continuous) {
                form->priv->slot_container = (GtkContainer*) gtk_vbox_new (TRUE, 0);

		/* just so slot_container doesn't have to expand and fill the display */
		GtkBox *box = (GtkBox*) gtk_vbox_new(FALSE, 0);
		gtk_box_pack_start ( box,
				     (GtkWidget*) form->priv->slot_container, 
				     FALSE, FALSE, 0);
		gtk_box_pack_start ( box,
				     gtk_vbox_new (TRUE, 0),
				     TRUE, TRUE, 0);

		GtkScrolledWindow *scrolled = (GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL);
                gtk_scrolled_window_add_with_viewport (scrolled,
						       (GtkWidget*) box);
		gtk_scrolled_window_set_policy (scrolled, 
						GTK_POLICY_AUTOMATIC,
						GTK_POLICY_AUTOMATIC);
                form->priv->display_widget = (GtkContainer*) scrolled;

		/* fill in with slots */
                const gint n = gda_data_model_get_n_rows (form->priv->data_model);
		gint i;
                for (i=0; i<n; i++) {
                        GnomeDbFormSlot *slot = form_slot_new (form, i);
			gtk_box_pack_start (GTK_BOX (slot), 
					    gtk_hseparator_new(), 
					    FALSE, FALSE, 2);
                        gtk_box_pack_start ( (GtkBox*) form->priv->slot_container,
					     (GtkWidget*) slot,
					     FALSE, FALSE, 0);
                }
	} else {
                form->priv->display_widget = (GtkContainer*) gtk_vbox_new (FALSE, 0);
		GnomeDbFormSlot *slot = form_slot_new (form, 0);
		gtk_box_pack_start ( (GtkBox*) form->priv->display_widget, 
				     (GtkWidget*) slot,
				     TRUE, TRUE, 0);
		form->priv->slot_container = GTK_CONTAINER(slot);
		gtk_adjustment_set_value (form->priv->adjustment, 0.0);
		gtk_box_pack_start ( (GtkBox*) form->priv->display_widget, 
				     (GtkWidget*) form->priv->page_flipper,
				     FALSE, FALSE, 0);
		/* will have to reconnect when gnome_db_form_add_page_flipper() is called */
		form->priv->adjustment_value_changed_handler_id
			= g_signal_connect ( (GtkObject*) form->priv->adjustment,
					     "value-changed",
					     GTK_SIGNAL_FUNC (row_change_callback),
					     slot);
        }
	gnome_db_form_sync_widgets (form);
	gtk_box_pack_start (GTK_BOX (form), (GtkWidget*) form->priv->display_widget, TRUE, TRUE, 0);
	gtk_widget_show_all ( (GtkWidget*) form->priv->display_widget);	
}


void
destroy_displaying_widgets (GnomeDbForm *form)
{
	g_printerr ("\ndestroy_displaying_widgets() ");
	g_assert (GNOME_DB_IS_FORM (form));

	if (! form->priv->continuous && form->priv->adjustment_value_changed_handler_id) {
		g_signal_handler_disconnect (form->priv->adjustment,
					     form->priv->adjustment_value_changed_handler_id);
		form->priv->adjustment_value_changed_handler_id = 0;
	}
	if (form->priv->display_widget) {
		/* gtk_widget_destroy seems to destroys page_flipper eventhough
		   we have called g_object_ref() on it, so keep the widget out of
		   bulldozer's way */
		if (! form->priv->continuous) {
			gtk_container_remove (form->priv->display_widget,
					      (GtkWidget*) form->priv->page_flipper);
		}
		gtk_widget_destroy (GTK_WIDGET (form->priv->display_widget));
	}

	form->priv->display_widget = NULL;
	form->priv->slot_container = NULL; /* destroyed as a child of display_widget */
}



void
gdm_changed_callback (GnomeDbForm *form)
{
/* FIXME:
	*** update (min, max), check that val is ok
	*** if gdarow get shuffled, stick with gdarow and update range_widget value
	*** if gdarow is gnone, go to the next one (the user might be deleting recs)		 
	*/
}

void
link_model (GnomeDbForm *form, GdaDataModel *model)
{
	gint i,n;
	GdaFieldAttributes **a;

	g_return_if_fail (GNOME_DB_IS_FORM (form));
	g_return_if_fail (GDA_IS_DATA_MODEL (model));

	g_object_ref (G_OBJECT (model));
	form->priv->data_model = model;

	n = gda_data_model_get_n_columns(model);
	a = (GdaFieldAttributes **) g_new (gpointer, n);
	for (i=0; i<n; i++)
		a[i] = gda_data_model_describe_column (form->priv->data_model, i);
	form->priv->field_attributes = a;

	form->priv->gdm_changed_handler_id 
		= g_signal_connect (model, 
				    "changed",
				    GTK_SIGNAL_FUNC(gdm_changed_callback),
				    form);
	g_signal_emit (G_OBJECT (form), form_signals[MODEL_CHANGED], 0);
}

void
unlink_model (GnomeDbForm *form)
{
	gint i,n;
	GdaFieldAttributes **a;

	g_return_if_fail (GNOME_DB_IS_FORM (form));
	g_return_if_fail (GDA_IS_DATA_MODEL (form->priv->data_model));

	a = form->priv->field_attributes;
	n = gda_data_model_get_n_columns(form->priv->data_model);
	for (i=0; i<n; i++)
		g_free (a[i]);
	g_free (a);
	form->priv->field_attributes = NULL;

	g_assert (form->priv->gdm_changed_handler_id != 0);
	g_signal_handler_disconnect (form->priv->data_model,
				     form->priv->gdm_changed_handler_id);
	g_object_unref (G_OBJECT (form->priv->data_model));
}


/* == public functions ========================================= */

/**
 * gnome_db_form_set_model
 * @form: a #GnomeDbForm widget.
 * @model: the data model to associate the widget with.
 */
void
gnome_db_form_set_model (GnomeDbForm *form, GdaDataModel *model)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));
	g_return_if_fail (GDA_IS_DATA_MODEL (model));

	destroy_displaying_widgets (form);

	// change data_model
	unlink_model (form);
	link_model (form, model);

	// get display back up
	gint n = gda_data_model_get_n_rows (model);
	gtk_adjustment_set_value (form->priv->adjustment, 0.0);
	form->priv->adjustment->upper = (double) (n-1);
	gtk_adjustment_changed (form->priv->adjustment);
	create_displaying_widgets (form);
}


/**
 * gnome_db_form_get_model
 * @form: a #GnomeDbForm widget.
 *
 * Return the #GdaDataModel being used by the given widget.
 */
GdaDataModel *
gnome_db_form_get_model (GnomeDbForm *form)
{
	g_return_val_if_fail (GNOME_DB_IS_FORM (form), NULL);
	return form->priv->data_model;
}


extern void	gnome_db_form_slot_sync_widgets (GnomeDbFormSlot *form_slot);
void
gnome_db_form_sync_widgets (GnomeDbForm *form)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));

	if (form->priv->continuous) {
		GList *list = gtk_container_get_children (form->priv->slot_container);
		while (list) {
			gnome_db_form_slot_sync_widgets (list->data);
			list = list->next;
		}
	} else {
		GList *list = gtk_container_get_children (form->priv->display_widget);
		gnome_db_form_slot_sync_widgets (list->data); /* just the first one */
	}
}


/**
 * gnome_db_form_set_current_row
 * @form: a #GnomeDbForm widget.
 * @row: the new row to display.
 *
 * Sets the current row on the given form widget.
 */
void
gnome_db_form_set_current_row (GnomeDbForm *form, gint row)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));
	g_return_if_fail (row >= 0);
	g_return_if_fail (row < gda_data_model_get_n_rows (form->priv->data_model));
	g_return_if_fail (! form->priv->continuous);

	gtk_adjustment_set_value (form->priv->adjustment, row);
	/* signals take care of the rest */
}


/**
 * gnome_db_form_get_current_row
 * @form: a #GnomeDbForm widget.
 *
 * Get the row currently being shown (navigator form) or currently selected
 * (list form) by the given form widget.
 *
 * Returns: the current row.
 */
gint
gnome_db_form_get_current_row (GnomeDbForm *form)
{
	g_return_val_if_fail (GNOME_DB_IS_FORM (form), -1);

	if (form->priv->continuous)
		return -1;
	else 
		return rint (gtk_adjustment_get_value(form->priv->adjustment));
}


/**
 * gnome_db_form_set_form_type
 */
void
gnome_db_form_set_form_type (GnomeDbForm *form, FORM_TYPE form_type)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));

	destroy_displaying_widgets (form);
	form->priv->form_type = form_type;
	create_displaying_widgets (form);
}



/**
 * gnome_db_form_get_form_type
 */
FORM_TYPE
gnome_db_form_get_form_type (GnomeDbForm *form)
{
	g_return_val_if_fail (GNOME_DB_IS_FORM (form), NULL);
	return form->priv->form_type;
}


/**
 * gnome_db_form_add_page_flipper
 * @form: a #GnomeDbForm
 * @page_flipper: user's #Widget to be used as the form's page flipper
 * @adjustment: #Adjustment associated with page_flipper (see #GtkRange for example).
 * 
 * Set the page flipper widget of a form.  The form will take care of referencing
 * and destroying page_flipper.
 **/
void
gnome_db_form_add_page_flipper (GnomeDbForm   *form, 
				GtkWidget     *page_flipper,
				GtkAdjustment *adjustment)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));
	g_return_if_fail (GTK_IS_WIDGET (page_flipper));
	g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));

	if (form->priv->page_flipper) {
		gtk_container_remove (form->priv->display_widget,
				      (GtkWidget*) form->priv->page_flipper);
		g_signal_handler_disconnect (form->priv->adjustment,
					     form->priv->adjustment_value_changed_handler_id);
		form->priv->adjustment_value_changed_handler_id = 0;
		g_object_unref (form->priv->page_flipper);
	}

	g_object_ref (page_flipper);
	gtk_object_sink ( (GtkObject*)page_flipper);
	if (form->priv->continuous && form->priv->display_widget) {
		gtk_container_add ( (GtkContainer*) form->priv->display_widget, 
				    (GtkWidget*) page_flipper);
		form->priv->adjustment_value_changed_handler_id
			= g_signal_connect ( (GtkObject*) adjustment,
					     "value-changed",
					     GTK_SIGNAL_FUNC (row_change_callback),
					     form->priv->slot_container);
	}
	form->priv->page_flipper = page_flipper;
	form->priv->adjustment   = adjustment;
}

GtkWidget*
gnome_db_form_get_page_flipper (GnomeDbForm *form) {
	g_return_if_fail (GNOME_DB_IS_FORM (form));

	return form->priv->page_flipper;
}


void
gnome_db_form_set_continuous (GnomeDbForm *form, gboolean continuous)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));

	destroy_displaying_widgets (form);
	form->priv->continuous = continuous;
	create_displaying_widgets (form);
}


gboolean
gnome_db_form_get_continuous (GnomeDbForm *form)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));
	return form->priv->continuous;
}


/*
 * Setting to FALSE makes all the bound widgets non-interactive, which
 * prevents end users from editing or saving from hitting Enter key.
 * If your code call various save functions, they still save.
 */
void
gnome_db_form_set_editable (GnomeDbForm *form, gboolean editable)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));
	if (form->priv->editable = editable)
	form->priv->editable = editable;
	if (form->priv->continuous) {
		GList *l = gtk_container_get_children(form->priv->slot_container);
		while (l) {
			gnome_db_form_slot_set_editable ( (GnomeDbFormSlot*) l->data, editable);
			l = l->next;
		}
	} else
		gnome_db_form_slot_set_editable ( (GnomeDbFormSlot*) form->priv->slot_container, editable);
}

gboolean
gnome_db_form_get_editable (GnomeDbForm *form)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));
	return form->priv->editable;
}


void
gnome_db_form_set_insertable (GnomeDbForm *form, gboolean insertable)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));
	
	if (form->priv->insertable == insertable)  return;

	form->priv->insertable = insertable;
	/* update page flipper regardless of priv->continuous */
	form->priv->adjustment->lower = insertable ? -1:0;
	gtk_adjustment_changed (form->priv->adjustment);

	if (form->priv->continuous && insertable) {
		/* FIXME: 
		   add a slot to form->priv->slot_container
		 */
	}
	if (form->priv->continuous && !insertable) {
		/* FIXME:
		   check if user is editing the last(new) slot
		   take off that last slot
		*/
	}
}


gboolean
gnome_db_form_get_insertable (GnomeDbForm *form)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));
	return form->priv->insertable;
}

/*
void
gnome_db_form_set_edited (GnomeDbForm *form)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));
	form->priv->edited = TRUE;
	g_signal_emit (form, form_signals[EDITED], 0);
}
*/
gboolean
check_edited (GnomeDbForm *form)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));

	// form->priv->edited = 
	gboolean ret;
	if (form->priv->continuous) {
		ret = FALSE;
		GList *l = gtk_container_get_children(form->priv->slot_container);
		while (l) {
			if ( gnome_db_form_slot_get_edited ( (GnomeDbFormSlot*) l->data) ) {
				ret = TRUE;
				break;
			}
			l = l->next;
		}
	} else
		ret = gnome_db_form_slot_get_edited ( (GnomeDbFormSlot*) form->priv->slot_container);
	return ret;
}
gboolean
gnome_db_form_get_edited (GnomeDbForm *form)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));
	return check_edited (form);
//	return form->priv->edited;
}

/* return whether succeed 
 */
gboolean
gnome_db_form_save_all (GnomeDbForm *form)
{
	g_return_if_fail (GNOME_DB_IS_FORM (form));

	gboolean ret;
	if (form->priv->continuous) {
		ret = TRUE;
		GList *l = gtk_container_get_children(form->priv->slot_container);
		while (l) {
			ret = ret && gnome_db_form_slot_save ( (GnomeDbFormSlot*) l->data);
			l = l->next;
		}
	} else
		ret = gnome_db_form_slot_save ( (GnomeDbFormSlot*) form->priv->slot_container);
	return ret;
}


/* == the build-in default form_type's ============================= */

void
_GNOME_DB_FORM_TYPE_NAVIGATOR (GnomeDbFormSlot *form_slot)
{
	GtkWidget *navigator;
	gint cols;
	GdaDataModel *model;

	g_return_if_fail (GNOME_DB_IS_FORM_SLOT (form_slot));
	model = gnome_db_form_slot_get_data_model(form_slot);
	if (model==NULL)  return;
	g_return_if_fail (GDA_IS_DATA_MODEL (model));

	cols = gda_data_model_get_n_columns (model);
	navigator = (GtkWidget*) gtk_table_new (cols, 2, FALSE);
	// gtk_table_set_row_spacings ((GtkTable*) navigator, 3);
        // gtk_table_set_col_spacings ((GtkTable*) navigator, 3);

	/* create the entry widgets for each field */
	gint col;
	for (col = 0; col < cols; col++) {
		const gchar  *name;
		GtkWidget *label;
		GtkWidget *entry;
		GdaFieldAttributes *fa;

		fa = gda_data_model_describe_column (model, col);

		name = gda_field_attributes_get_name (fa);
		label = (GtkWidget*) gnome_db_new_label_widget (name);
		gtk_table_attach (GTK_TABLE (navigator), label, 0, 1, col, col + 1,
				  GTK_FILL, GTK_FILL, 2, 0);

		switch (gda_field_attributes_get_gdatype (fa)) {
		case GDA_VALUE_TYPE_BOOLEAN :
			entry = gtk_check_button_new();
			gtk_widget_set_sensitive (entry, FALSE);
			gnome_db_form_slot_bind_toggle_button (form_slot, name, (GtkToggleButton*) entry);
			break;
		/* FIXME: This should be a hex editor
		case GDA_VALUE_TYPE_BINARY :
			entry = gtk_text_view_new ();
			gtk_text_view_set_editable ( (GtkTextView*) entry, FALSE);
			gnome_db_form_slot_bind_text_view (form_slot, name, (GtkTextView*) entry);
			break;
		*/
		default :
			entry = gtk_entry_new ();
			gtk_editable_set_editable ( (GtkEditable*) entry, FALSE);

			gnome_db_form_slot_bind_entry (form_slot, name, (GtkEntry*) entry);
			break;
		}

		gtk_table_attach (GTK_TABLE (navigator), entry, 1, 2, col, col + 1,
				  GTK_FILL, GTK_FILL, 2, 0);

		gda_field_attributes_free (fa);
	}
	gtk_container_add ( (GtkContainer*) form_slot, (GtkWidget*) navigator);

}


void
_GNOME_DB_FORM_TYPE_LIST (GnomeDbFormSlot *form_slot)
{
	// FIXME:
	GtkWidget *label = gtk_label_new("To be implemented");
	gtk_container_add ( (GtkContainer*) form_slot, label); 
}





// ================
GdaValueType
get_type (GnomeDbForm *form, gint col) 
{
	return form->priv->field_attributes[col]->gda_type;
};
