/* GNOME DB library
 * Copyright (C) 1999-2002 The GNOME Foundation.
 *
 * AUTHORS:
 *      Rodrigo Moya <rodrigo@gnome-db.org>
 *
 * This Library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this Library; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#if !defined(__gnome_db_form_h__)
#  define __gnome_db_form_h__

// #include <glib.h>
// #include <glib-object.h>
#include <libgda/gda-data-model.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkrange.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtkentry.h>
#include <gtk/gtktextview.h>


G_BEGIN_DECLS


#define GNOME_DB_TYPE_FORM            (gnome_db_form_get_type())
#define GNOME_DB_FORM(obj)            (G_TYPE_CHECK_INSTANCE_CAST (obj, GNOME_DB_TYPE_FORM, GnomeDbForm))
#define GNOME_DB_FORM_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST (klass, GNOME_DB_TYPE_FORM, GnomeDbFormClass))
#define GNOME_DB_IS_FORM(obj)         (G_TYPE_CHECK_INSTANCE_TYPE (obj, GNOME_DB_TYPE_FORM))
#define GNOME_DB_IS_FORM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GNOME_DB_TYPE_FORM))

typedef struct _GnomeDbForm        GnomeDbForm;
typedef struct _GnomeDbFormClass   GnomeDbFormClass;
typedef struct _GnomeDbFormPrivate GnomeDbFormPrivate;



#define GNOME_DB_TYPE_FORM_SLOT            (gnome_db_form_slot_get_type())
#define GNOME_DB_FORM_SLOT(obj)            (G_TYPE_CHECK_INSTANCE_CAST (obj, GNOME_DB_TYPE_FORM_SLOT, GnomeDbFormSlot))
#define GNOME_DB_FORM_SLOT_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST (klass, GNOME_DB_TYPE_FORM_SLOT, GnomeDbFormSlotClass))
#define GNOME_DB_IS_FORM_SLOT(obj)         (G_TYPE_CHECK_INSTANCE_TYPE (obj, GNOME_DB_TYPE_FORM_SLOT))
#define GNOME_DB_IS_FORM_SLOT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GNOME_DB_TYPE_FORM_SLOT))

typedef struct _GnomeDbFormSlot        GnomeDbFormSlot;
typedef struct _GnomeDbFormSlotClass   GnomeDbFormSlotClass;
typedef struct _GnomeDbFormSlotPrivate GnomeDbFormSlotPrivate;




/* == GnomeDbForm ============== */

struct _GnomeDbForm {
	GtkVBox vbox;	/* FIXME: change this to GtkWidget and handle graphics */
	GnomeDbFormPrivate *priv;
};

struct _GnomeDbFormClass {
	GtkVBoxClass parent_class;

	/* signals */
	void (* model_changed) (GnomeDbForm *form);
	void (* edited) (GnomeDbForm *form);
};


/**
 * FORM_TYPE
 * @slot: a #GnomeDbFormSlot
 * 
 * A function that populate slot (a subclass of #GtkVBox) with widgets
 * that make up a form.  In addition, widgets with a call to
 * #gnomedb_form_slot_bind(slot, ...) will be bound to a column in #GdaDataModel.
 **/
typedef void (*FORM_TYPE) (GnomeDbFormSlot* slot);

GType            gnome_db_form_get_type (void);
GtkWidget       *gnome_db_form_new (void);
/*
GtkWidget       *gnome_db_form_new_with_args (GdaDataModel *model,
					      FORM_TYPE form_type,
					      gboolean continuous);
*/

void             gnome_db_form_set_model (GnomeDbForm *form, GdaDataModel *model);
GdaDataModel    *gnome_db_form_get_model (GnomeDbForm *form);

void             gnome_db_form_set_current_row (GnomeDbForm *form, gint row);
gint             gnome_db_form_get_current_row (GnomeDbForm *form);

void             gnome_db_form_set_form_type (GnomeDbForm *form, FORM_TYPE form_type);
FORM_TYPE        gnome_db_form_get_form_type (GnomeDbForm *form);

void		 gnome_db_form_add_page_flipper (GnomeDbForm *form, 
						 GtkWidget *range,
						 GtkAdjustment *adjustment);
GtkWidget	*gnome_db_form_get_page_flipper (GnomeDbForm *form);
/*		 gnome_db_set_page_flipper_pos   */

void             gnome_db_form_set_continuous (GnomeDbForm *form, gboolean continuous);
gboolean         gnome_db_form_get_continuous (GnomeDbForm *form);

void		 gnome_db_form_set_editable (GnomeDbForm *form, gboolean editable);
gboolean	 gnome_db_form_get_editable (GnomeDbForm *form);
//   this will also depend on data_model

void             gnome_db_form_set_insertable (GnomeDbForm *form, gboolean insertable);
gboolean         gnome_db_form_get_insertable (GnomeDbForm *form);


// void		gnome_db_form_set_edited (GnomeDbForm *form);
gboolean         gnome_db_form_get_edited (GnomeDbForm *form);

gboolean         gnome_db_form_save_all (GnomeDbForm *form);
// FIXME, have page flipper call can check this before flipping pages

extern const FORM_TYPE GNOME_DB_FORM_TYPE_NAVIGATOR;
extern const FORM_TYPE GNOME_DB_FORM_TYPE_LIST;





/* == GnomeDbFormSlot ============== */

struct _GnomeDbFormSlot {
	GtkVBox vbox;
	GnomeDbFormSlotPrivate *priv;
};

struct _GnomeDbFormSlotClass {
        GtkVBoxClass parent_class;

	void (* default_sync_widgets) (GnomeDbFormSlot *form_slot);
	void (* save) (GnomeDbFormSlot *form_slot);
	void (* edit) (GnomeDbFormSlot *form_slot);

	// this need to be NULL at all time.  Probably should make private
	void (* null) (GnomeDbFormSlot *form_slot);
};


// return TRUE if succeed, FALSE if fail
typedef gboolean (*GET_WIDGET_GDA_VALUE) (const GtkWidget* widget, GdaValue* value);
typedef void (*SET_WIDGET_GDA_VALUE) (GtkWidget* widget, const GdaValue* value);
typedef void (*SET_EDITABLE) (GtkWidget* widget, gboolean editable);

GnomeDbForm     *gnome_db_form_slot_get_form (GnomeDbFormSlot *form_slot);
GdaDataModel    *gnome_db_form_slot_get_data_model (GnomeDbFormSlot *form_slot);
const GdaRow    *gnome_db_form_slot_get_gda_row (GnomeDbFormSlot *form_slot);
/* no get_row_number() b/c the GdaRow might not even be in the data_model
   (e.g. when creating new record)
*/

void		 gnome_db_form_slot_bind (GnomeDbFormSlot *form_slot, 
					  const gchar* column, 
					  void *instance,
					  GET_WIDGET_GDA_VALUE get_value, 
					  SET_WIDGET_GDA_VALUE set_value,
					  SET_EDITABLE set_editable);

void		 gnome_db_form_slot_set_edited (GnomeDbFormSlot *form_slot);
gboolean	 gnome_db_form_slot_get_edited (GnomeDbFormSlot *form_slot);

gboolean	 gnome_db_form_slot_save (GnomeDbFormSlot *form_slot);

/* FIXME 
 - GtkTipsQuery has a way of grabing all signals ... check it out
 - all displaying widgets has its own X-windows!
   see how entry was implemented..., how X events are handled
 - make Balloon binded to any widget
*/



/* from gnome-db-form-slot-bind-widgets.c */
void		 gnome_db_form_slot_bind_toggle_button (GnomeDbFormSlot *form_slot, 
							const gchar* column, 
							GtkToggleButton *toggle_button);
void		 gnome_db_form_slot_bind_entry (GnomeDbFormSlot *form_slot, 
						const gchar* column, 
						GtkEntry *entry);
void             gnome_db_form_slot_bind_text_view (GnomeDbFormSlot *form_slot,
						    const gchar* column, 
						    GtkTextView *entry);



G_END_DECLS

#endif
