
#include <gtk/gtk.h>
#include <libgda/libgda.h>
/* #include <libgnomedb/libgnomedb.h> */
#include "gnome-db-form.h"


/* Custom form type function
 * 
 * GnomeDbFormSlot: 
 *
 *    is a subclass of VBox.  In addition to containing widgets to be
 *    displayed, it let you bind a widget to a column of data_model.
 *
 * GnomeForm:
 *
 *    contains one FormSlot in non-continuous mode, and multiple
 *    FormSlot's in continuous mode.
 *
 */


/*  user-defined form type has two main jobs
 *    - call gnome_db_form_slot_bind* to make widgets data-bound
 *    - add widgets to GTK_VBox(form_slot) for displaying
 */
void
my_type (GnomeDbFormSlot *form_slot)
{
	GtkWidget *entry;
	
	entry = gtk_entry_new();
	// this make bind the widget to the column "a"
	gnome_db_form_slot_bind_entry (form_slot, "c_date", (GtkEntry*) entry);
	gtk_container_add ( (GtkContainer*) form_slot, entry); 

	// bind column "a"
	entry = gtk_entry_new();
	gnome_db_form_slot_bind_entry (form_slot, "b", (GtkEntry*) entry);
	gtk_container_add ( (GtkContainer*) form_slot, entry); 
}


void
change_type_callback (GtkButton* button, GnomeDbForm *form)
{
	static gint form_type = 0;

	// Save all user-edited data
	if (! gnome_db_form_save_all (form))  return;

	// switch between form types
	form_type = (form_type+1)%3;
	switch (form_type) {
		case 0:	gnome_db_form_set_form_type(form, GNOME_DB_FORM_TYPE_NAVIGATOR);
			break;
		case 1:	gnome_db_form_set_form_type(form, GNOME_DB_FORM_TYPE_LIST);
			break;
		case 2:	gnome_db_form_set_form_type(form, my_type);
			break;
	}
}


void
toggle_continuous_callback (GtkButton* button, GnomeDbForm *form)
{
	if (! gnome_db_form_save_all (form))  return;
	gnome_db_form_set_continuous (form, !gnome_db_form_get_continuous (form));
}





int
main (int argc, char *argv[])
{
	GtkWidget	*window;
	GtkBox		*vbox;
	GtkWidget	*button;
	GnomeDbForm	*form;

	// init
	gtk_init (&argc, &argv);
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (window), "GnomeDbForm Example");
	g_signal_connect (G_OBJECT (window), "destroy",
			  G_CALLBACK (exit), NULL);
	
	// create widgets
	vbox = (GtkBox*) gtk_vbox_new (FALSE, 4);
	gtk_container_add ( (GtkContainer*) window, (GtkWidget*) vbox);

	form = (GnomeDbForm *) gnome_db_form_new();
	gtk_box_pack_start (vbox, (GtkWidget*) form, TRUE, TRUE, 0);

	button = gtk_button_new_with_label ("Change Type!");
	gtk_box_pack_start (vbox, (GtkWidget*) button, FALSE, FALSE, 0);
	g_signal_connect (button, "clicked", (GCallback) change_type_callback, form);

	button = gtk_button_new_with_label ("Toggle Continuous");
	gtk_box_pack_start (vbox, (GtkWidget*) button, FALSE, FALSE, 0);
	g_signal_connect (button, "clicked", (GCallback) toggle_continuous_callback, form);

	button = gtk_button_new_with_label ("Just a Dummy Button");
	gtk_box_pack_start (vbox, (GtkWidget*) button, FALSE, FALSE, 0);

	// connect to data source
	GdaClient *client;
	GdaConnection *connection;
	gchar *buffer;
	GdaCommand *command;
	GdaDataModel *model;

	client = gda_client_new();
	connection = gda_client_open_connection (client,
						 "calvaris",
						 NULL,
						 NULL,
						 (GdaConnectionOptions) 0);
	buffer = "select * from a;";
	command = gda_command_new (buffer, GDA_COMMAND_TYPE_SQL,
				   GDA_COMMAND_OPTION_STOP_ON_ERRORS);
	model = gda_connection_execute_single_command (connection, command, NULL);
	gda_command_free (command);

	if (model) {
		gnome_db_form_set_editable(form, TRUE);
		gnome_db_form_set_model(form, model);
	}
	else 
		g_printerr ("No data model :-P\n");

	// that's it
	gtk_widget_show_all (window);
	gtk_main ();
  
	return 0;
}



  /*
  GnomeDbBrowser *dbbrowser = (GnomeDbBrowser *) gnome_db_browser_new();
  gnome_db_browser_set_connection(dbbrowser, connection);
  gtk_box_pack_start( GTK_BOX(vbox.gobj()), (GtkWidget *)dbbrowser, TRUE, TRUE, 0);
  //  gtk_widget_show( (GtkWidget *) dbbrowser);
  */


